# pyre-ignore-all-errors[14]
from __future__ import annotations

# DO NOT EDIT!
# This file was automatically generated by "generate_ids.py"

__all__ = ["unit_typeid", "ability_id", "upgrade_id", "buff_id", "effect_id"]
