from enum import Enum

class ActionResult(Enum):
    Success: int
    NotSupported: int
    Error: int
    CantQueueThatOrder: int
    Retry: int
    Cooldown: int
    QueueIsFull: int
    RallyQueueIsFull: int
    NotEnoughMinerals: int
    NotEnoughVespene: int
    NotEnoughTerrazine: int
    NotEnoughCustom: int
    NotEnoughFood: int
    FoodUsageImpossible: int
    NotEnoughLife: int
    NotEnoughShields: int
    NotEnoughEnergy: int
    LifeSuppressed: int
    ShieldsSuppressed: int
    EnergySuppressed: int
    NotEnoughCharges: int
    CantAddMoreCharges: int
    TooMuchMinerals: int
    TooMuchVespene: int
    TooMuchTerrazine: int
    TooMuchCustom: int
    TooMuchFood: int
    TooMuchLife: int
    TooMuchShields: int
    TooMuchEnergy: int
    MustTargetUnitWithLife: int
    MustTargetUnitWithShields: int
    MustTargetUnitWithEnergy: int
    CantTrade: int
    CantSpend: int
    CantTargetThatUnit: int
    CouldntAllocateUnit: int
    UnitCantMove: int
    TransportIsHoldingPosition: int
    BuildTechRequirementsNotMet: int
    CantFindPlacementLocation: int
    CantBuildOnThat: int
    CantBuildTooCloseToDropOff: int
    CantBuildLocationInvalid: int
    CantSeeBuildLocation: int
    CantBuildTooCloseToCreepSource: int
    CantBuildTooCloseToResources: int
    CantBuildTooFarFromWater: int
    CantBuildTooFarFromCreepSource: int
    CantBuildTooFarFromBuildPowerSource: int
    CantBuildOnDenseTerrain: int
    CantTrainTooFarFromTrainPowerSource: int
    CantLandLocationInvalid: int
    CantSeeLandLocation: int
    CantLandTooCloseToCreepSource: int
    CantLandTooCloseToResources: int
    CantLandTooFarFromWater: int
    CantLandTooFarFromCreepSource: int
    CantLandTooFarFromBuildPowerSource: int
    CantLandTooFarFromTrainPowerSource: int
    CantLandOnDenseTerrain: int
    AddOnTooFarFromBuilding: int
    MustBuildRefineryFirst: int
    BuildingIsUnderConstruction: int
    CantFindDropOff: int
    CantLoadOtherPlayersUnits: int
    NotEnoughRoomToLoadUnit: int
    CantUnloadUnitsThere: int
    CantWarpInUnitsThere: int
    CantLoadImmobileUnits: int
    CantRechargeImmobileUnits: int
    CantRechargeUnderConstructionUnits: int
    CantLoadThatUnit: int
    NoCargoToUnload: int
    LoadAllNoTargetsFound: int
    NotWhileOccupied: int
    CantAttackWithoutAmmo: int
    CantHoldAnyMoreAmmo: int
    TechRequirementsNotMet: int
    MustLockdownUnitFirst: int
    MustTargetUnit: int
    MustTargetInventory: int
    MustTargetVisibleUnit: int
    MustTargetVisibleLocation: int
    MustTargetWalkableLocation: int
    MustTargetPawnableUnit: int
    YouCantControlThatUnit: int
    YouCantIssueCommandsToThatUnit: int
    MustTargetResources: int
    RequiresHealTarget: int
    RequiresRepairTarget: int
    NoItemsToDrop: int
    CantHoldAnyMoreItems: int
    CantHoldThat: int
    TargetHasNoInventory: int
    CantDropThisItem: int
    CantMoveThisItem: int
    CantPawnThisUnit: int
    MustTargetCaster: int
    CantTargetCaster: int
    MustTargetOuter: int
    CantTargetOuter: int
    MustTargetYourOwnUnits: int
    CantTargetYourOwnUnits: int
    MustTargetFriendlyUnits: int
    CantTargetFriendlyUnits: int
    MustTargetNeutralUnits: int
    CantTargetNeutralUnits: int
    MustTargetEnemyUnits: int
    CantTargetEnemyUnits: int
    MustTargetAirUnits: int
    CantTargetAirUnits: int
    MustTargetGroundUnits: int
    CantTargetGroundUnits: int
    MustTargetStructures: int
    CantTargetStructures: int
    MustTargetLightUnits: int
    CantTargetLightUnits: int
    MustTargetArmoredUnits: int
    CantTargetArmoredUnits: int
    MustTargetBiologicalUnits: int
    CantTargetBiologicalUnits: int
    MustTargetHeroicUnits: int
    CantTargetHeroicUnits: int
    MustTargetRoboticUnits: int
    CantTargetRoboticUnits: int
    MustTargetMechanicalUnits: int
    CantTargetMechanicalUnits: int
    MustTargetPsionicUnits: int
    CantTargetPsionicUnits: int
    MustTargetMassiveUnits: int
    CantTargetMassiveUnits: int
    MustTargetMissile: int
    CantTargetMissile: int
    MustTargetWorkerUnits: int
    CantTargetWorkerUnits: int
    MustTargetEnergyCapableUnits: int
    CantTargetEnergyCapableUnits: int
    MustTargetShieldCapableUnits: int
    CantTargetShieldCapableUnits: int
    MustTargetFlyers: int
    CantTargetFlyers: int
    MustTargetBuriedUnits: int
    CantTargetBuriedUnits: int
    MustTargetCloakedUnits: int
    CantTargetCloakedUnits: int
    MustTargetUnitsInAStasisField: int
    CantTargetUnitsInAStasisField: int
    MustTargetUnderConstructionUnits: int
    CantTargetUnderConstructionUnits: int
    MustTargetDeadUnits: int
    CantTargetDeadUnits: int
    MustTargetRevivableUnits: int
    CantTargetRevivableUnits: int
    MustTargetHiddenUnits: int
    CantTargetHiddenUnits: int
    CantRechargeOtherPlayersUnits: int
    MustTargetHallucinations: int
    CantTargetHallucinations: int
    MustTargetInvulnerableUnits: int
    CantTargetInvulnerableUnits: int
    MustTargetDetectedUnits: int
    CantTargetDetectedUnits: int
    CantTargetUnitWithEnergy: int
    CantTargetUnitWithShields: int
    MustTargetUncommandableUnits: int
    CantTargetUncommandableUnits: int
    MustTargetPreventDefeatUnits: int
    CantTargetPreventDefeatUnits: int
    MustTargetPreventRevealUnits: int
    CantTargetPreventRevealUnits: int
    MustTargetPassiveUnits: int
    CantTargetPassiveUnits: int
    MustTargetStunnedUnits: int
    CantTargetStunnedUnits: int
    MustTargetSummonedUnits: int
    CantTargetSummonedUnits: int
    MustTargetUser1: int
    CantTargetUser1: int
    MustTargetUnstoppableUnits: int
    CantTargetUnstoppableUnits: int
    MustTargetResistantUnits: int
    CantTargetResistantUnits: int
    MustTargetDazedUnits: int
    CantTargetDazedUnits: int
    CantLockdown: int
    CantMindControl: int
    MustTargetDestructibles: int
    CantTargetDestructibles: int
    MustTargetItems: int
    CantTargetItems: int
    NoCalldownAvailable: int
    WaypointListFull: int
    MustTargetRace: int
    CantTargetRace: int
    MustTargetSimilarUnits: int
    CantTargetSimilarUnits: int
    CantFindEnoughTargets: int
    AlreadySpawningLarva: int
    CantTargetExhaustedResources: int
    CantUseMinimap: int
    CantUseInfoPanel: int
    OrderQueueIsFull: int
    CantHarvestThatResource: int
    HarvestersNotRequired: int
    AlreadyTargeted: int
    CantAttackWeaponsDisabled: int
    CouldntReachTarget: int
    TargetIsOutOfRange: int
    TargetIsTooClose: int
    TargetIsOutOfArc: int
    CantFindTeleportLocation: int
    InvalidItemClass: int
    CantFindCancelOrder: int
